@@ -1,8 +1,15 @@
 import json
-import os  # noqa
-import pathlib  # noqa
-import sys
-from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, Tuple, Union
+import os
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Awaitable,
+    Callable,
+    Iterable,
+    Mapping,
+    Tuple,
+    Union,
+)
 
 from multidict import CIMultiDict, CIMultiDictProxy, MultiDict, MultiDictProxy, istr
 from yarl import URL
@@ -10,12 +17,14 @@ from yarl import URL
 DEFAULT_JSON_ENCODER = json.dumps
 DEFAULT_JSON_DECODER = json.loads
 
-if TYPE_CHECKING:  # pragma: no cover
+if TYPE_CHECKING:
     _CIMultiDict = CIMultiDict[str]
     _CIMultiDictProxy = CIMultiDictProxy[str]
     _MultiDict = MultiDict[str]
     _MultiDictProxy = MultiDictProxy[str]
-    from http.cookies import BaseCookie, Morsel  # noqa
+    from http.cookies import BaseCookie, Morsel
+
+    from .web import Request, StreamResponse
 else:
     _CIMultiDict = CIMultiDict
     _CIMultiDictProxy = CIMultiDictProxy
@@ -39,8 +48,7 @@ LooseCookies = Union[
     "BaseCookie[str]",
 ]
 
+Handler = Callable[["Request"], Awaitable["StreamResponse"]]
+Middleware = Callable[["Request", Handler], Awaitable["StreamResponse"]]
 
-if sys.version_info >= (3, 6):
-    PathLike = Union[str, "os.PathLike[str]"]
-else:
-    PathLike = Union[str, pathlib.PurePath]
+PathLike = Union[str, "os.PathLike[str]"]
