@@ -9,6 +9,7 @@ import ssl
 import sys
 import uuid
 from collections import deque
+from contextlib import closing
 from unittest import mock
 
 import pytest
@@ -19,7 +20,6 @@ from aiohttp import client, web
 from aiohttp.client import ClientRequest, ClientTimeout
 from aiohttp.client_reqrep import ConnectionKey
 from aiohttp.connector import Connection, TCPConnector, _DNSCacheTable
-from aiohttp.helpers import PY_37
 from aiohttp.locks import EventResultOrError
 from aiohttp.test_utils import make_mocked_coro, unused_port
 from aiohttp.tracing import Trace
@@ -28,25 +28,19 @@ from aiohttp.tracing import Trace
 @pytest.fixture()
 def key():
     # Connection key
-    return ConnectionKey("localhost", 80, False, None, None, None, None)
+    return ConnectionKey("localhost", 80, False, True, None, None, None)
 
 
 @pytest.fixture
 def key2():
     # Connection key
-    return ConnectionKey("localhost", 80, False, None, None, None, None)
+    return ConnectionKey("localhost", 80, False, True, None, None, None)
 
 
 @pytest.fixture
 def ssl_key():
     # Connection key
-    return ConnectionKey("localhost", 80, True, None, None, None, None)
-
-
-@pytest.fixture
-def unix_sockname(shorttmpdir):
-    sock_path = shorttmpdir / "socket.sock"
-    return str(sock_path)
+    return ConnectionKey("localhost", 80, True, True, None, None, None)
 
 
 @pytest.fixture
@@ -84,7 +78,11 @@ def named_pipe_server(proactor_loop, pipe_name):
 
 
 def create_mocked_conn(conn_closing_result=None, **kwargs):
-    loop = asyncio.get_event_loop()
+    try:
+        loop = asyncio.get_running_loop()
+    except RuntimeError:
+        loop = asyncio.get_event_loop_policy().get_event_loop()
+
     proto = mock.Mock(**kwargs)
     proto.closed = loop.create_future()
     proto.closed.set_result(conn_closing_result)
@@ -244,7 +242,7 @@ async def test_create_conn(loop) -> None:
         await conn._create_connection(object(), [], object())
 
 
-async def test_context_manager(loop) -> None:
+async def test_connector_context_manager(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
 
     with pytest.warns(DeprecationWarning):
@@ -253,6 +251,8 @@ async def test_context_manager(loop) -> None:
 
     assert conn.closed
 
+    await conn.close()
+
 
 async def test_async_context_manager(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
@@ -269,7 +269,7 @@ async def test_close(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     assert not conn.closed
     conn._conns[("host", 8080, False)] = [(proto, object())]
-    conn.close()
+    await conn.close()
 
     assert not conn._conns
     assert proto.close.called
@@ -283,7 +283,7 @@ async def test_get(loop) -> None:
     proto = mock.Mock()
     conn._conns[1] = [(proto, loop.time())]
     assert conn._get(1) == proto
-    conn.close()
+    await conn.close()
 
 
 async def test_get_unconnected_proto(loop) -> None:
@@ -327,7 +327,7 @@ async def test_get_expired(loop) -> None:
     conn._conns[key] = [(proto, loop.time() - 1000)]
     assert conn._get(key) is None
     assert not conn._conns
-    conn.close()
+    await conn.close()
 
 
 async def test_get_expired_ssl(loop) -> None:
@@ -341,7 +341,7 @@ async def test_get_expired_ssl(loop) -> None:
     assert conn._get(key) is None
     assert not conn._conns
     assert conn._cleanup_closed_transports == [transport]
-    conn.close()
+    await conn.close()
 
 
 async def test_release_acquired(loop, key) -> None:
@@ -360,7 +360,7 @@ async def test_release_acquired(loop, key) -> None:
     assert 0 == len(conn._acquired)
     assert 0 == len(conn._acquired_per_host)
 
-    conn.close()
+    await conn.close()
 
 
 async def test_release_acquired_closed(loop, key) -> None:
@@ -375,7 +375,7 @@ async def test_release_acquired_closed(loop, key) -> None:
     assert 1 == len(conn._acquired)
     assert 1 == len(conn._acquired_per_host[key])
     assert not conn._release_waiter.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release(loop, key) -> None:
@@ -393,7 +393,7 @@ async def test_release(loop, key) -> None:
     assert conn._conns[key][0][0] == proto
     assert conn._conns[key][0][1] == pytest.approx(loop.time(), abs=0.1)
     assert not conn._cleanup_closed_transports
-    conn.close()
+    await conn.close()
 
 
 async def test_release_ssl_transport(loop, ssl_key) -> None:
@@ -407,7 +407,7 @@ async def test_release_ssl_transport(loop, ssl_key) -> None:
 
     conn._release(ssl_key, proto, should_close=True)
     assert conn._cleanup_closed_transports == [transport]
-    conn.close()
+    await conn.close()
 
 
 async def test_release_already_closed(loop) -> None:
@@ -416,7 +416,7 @@ async def test_release_already_closed(loop) -> None:
     proto = mock.Mock()
     key = 1
     conn._acquired.add(proto)
-    conn.close()
+    await conn.close()
 
     conn._release_waiters = mock.Mock()
     conn._release_acquired = mock.Mock()
@@ -435,7 +435,7 @@ async def test_release_waiter_no_limit(loop, key, key2) -> None:
     conn._release_waiter()
     assert len(conn._waiters[key]) == 0
     assert w.done.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release_waiter_first_available(loop, key, key2) -> None:
@@ -452,7 +452,7 @@ async def test_release_waiter_first_available(loop, key, key2) -> None:
         or not w1.set_result.called
         and w2.set_result.called
     )
-    conn.close()
+    await conn.close()
 
 
 async def test_release_waiter_release_first(loop, key, key2) -> None:
@@ -464,7 +464,7 @@ async def test_release_waiter_release_first(loop, key, key2) -> None:
     conn._release_waiter()
     assert w1.set_result.called
     assert not w2.set_result.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release_waiter_skip_done_waiter(loop, key, key2) -> None:
@@ -476,7 +476,7 @@ async def test_release_waiter_skip_done_waiter(loop, key, key2) -> None:
     conn._release_waiter()
     assert not w1.set_result.called
     assert w2.set_result.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release_waiter_per_host(loop, key, key2) -> None:
@@ -491,7 +491,7 @@ async def test_release_waiter_per_host(loop, key, key2) -> None:
     assert (w1.set_result.called and not w2.set_result.called) or (
         not w1.set_result.called and w2.set_result.called
     )
-    conn.close()
+    await conn.close()
 
 
 async def test_release_waiter_no_available(loop, key, key2) -> None:
@@ -504,7 +504,7 @@ async def test_release_waiter_no_available(loop, key, key2) -> None:
     conn._release_waiter()
     assert len(conn._waiters) == 1
     assert not w.done.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release_close(loop, key) -> None:
@@ -556,6 +556,36 @@ async def test_tcp_connector_certificate_error(loop) -> None:
     assert isinstance(ctx.value, aiohttp.ClientSSLError)
 
 
+async def test_tcp_connector_server_hostname_default(loop) -> None:
+    conn = aiohttp.TCPConnector(loop=loop)
+
+    with mock.patch.object(
+        conn._loop, "create_connection", autospec=True, spec_set=True
+    ) as create_connection:
+        create_connection.return_value = mock.Mock(), mock.Mock()
+
+        req = ClientRequest("GET", URL("https://127.0.0.1:443"), loop=loop)
+
+        with closing(await conn.connect(req, [], ClientTimeout())):
+            assert create_connection.call_args.kwargs["server_hostname"] == "127.0.0.1"
+
+
+async def test_tcp_connector_server_hostname_override(loop) -> None:
+    conn = aiohttp.TCPConnector(loop=loop)
+
+    with mock.patch.object(
+        conn._loop, "create_connection", autospec=True, spec_set=True
+    ) as create_connection:
+        create_connection.return_value = mock.Mock(), mock.Mock()
+
+        req = ClientRequest(
+            "GET", URL("https://127.0.0.1:443"), loop=loop, server_hostname="localhost"
+        )
+
+        with closing(await conn.connect(req, [], ClientTimeout())):
+            assert create_connection.call_args.kwargs["server_hostname"] == "localhost"
+
+
 async def test_tcp_connector_multiple_hosts_errors(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
 
@@ -657,7 +687,7 @@ async def test_tcp_connector_multiple_hosts_errors(loop) -> None:
 
     conn._loop.create_connection = create_connection
 
-    await conn.connect(req, [], ClientTimeout())
+    established_connection = await conn.connect(req, [], ClientTimeout())
     assert ips == ips_tried
 
     assert os_error
@@ -666,6 +696,8 @@ async def test_tcp_connector_multiple_hosts_errors(loop) -> None:
     assert fingerprint_error
     assert connected
 
+    established_connection.close()
+
 
 async def test_tcp_connector_resolve_host(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop, use_dns_cache=True)
@@ -761,7 +793,7 @@ async def test_tcp_connector_dns_throttle_requests_cancelled_when_close(
         f = loop.create_task(conn._resolve_host("localhost", 8080))
 
         await asyncio.sleep(0)
-        conn.close()
+        await conn.close()
 
         with pytest.raises(asyncio.CancelledError):
             await f
@@ -976,7 +1008,7 @@ async def test_release_close_do_not_delete_existing_connections(key) -> None:
     conn._release(key, proto)
     assert conn._conns[key] == [(proto1, 1)]
     assert proto.close.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release_not_started(loop) -> None:
@@ -990,7 +1022,7 @@ async def test_release_not_started(loop) -> None:
     assert rec[0][0] == proto
     assert rec[0][1] == pytest.approx(loop.time(), abs=0.05)
     assert not proto.close.called
-    conn.close()
+    await conn.close()
 
 
 async def test_release_not_opened(loop, key) -> None:
@@ -1070,7 +1102,7 @@ async def test_close_during_connect(loop) -> None:
 
     task = loop.create_task(conn.connect(req, None, ClientTimeout()))
     await asyncio.sleep(0)
-    conn.close()
+    await conn.close()
 
     fut.set_result(proto)
     with pytest.raises(aiohttp.ClientConnectionError):
@@ -1139,7 +1171,7 @@ async def test_cleanup2() -> None:
 
     assert conn._cleanup_handle is not None
     loop.call_at.assert_called_with(310, mock.ANY, mock.ANY)
-    conn.close()
+    await conn.close()
 
 
 async def test_cleanup3(key) -> None:
@@ -1157,7 +1189,7 @@ async def test_cleanup3(key) -> None:
 
     assert conn._cleanup_handle is not None
     loop.call_at.assert_called_with(319, mock.ANY, mock.ANY)
-    conn.close()
+    await conn.close()
 
 
 async def test_cleanup_closed(loop, mocker) -> None:
@@ -1187,9 +1219,9 @@ async def test_cleanup_closed_disabled(loop, mocker) -> None:
     assert not conn._cleanup_closed_transports
 
 
-async def test_tcp_connector_ctor(loop) -> None:
-    conn = aiohttp.TCPConnector(loop=loop)
-    assert conn._ssl is None
+async def test_tcp_connector_ctor() -> None:
+    conn = aiohttp.TCPConnector()
+    assert conn._ssl is True
 
     assert conn.use_dns_cache
     assert conn.family == 0
@@ -1262,7 +1294,7 @@ async def test___get_ssl_context1(loop) -> None:
 
 
 async def test___get_ssl_context2(loop) -> None:
-    ctx = ssl.SSLContext()
+    ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     conn = aiohttp.TCPConnector(loop=loop)
     req = mock.Mock()
     req.is_ssl.return_value = True
@@ -1271,16 +1303,16 @@ async def test___get_ssl_context2(loop) -> None:
 
 
 async def test___get_ssl_context3(loop) -> None:
-    ctx = ssl.SSLContext()
+    ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     conn = aiohttp.TCPConnector(loop=loop, ssl=ctx)
     req = mock.Mock()
     req.is_ssl.return_value = True
-    req.ssl = None
+    req.ssl = True
     assert conn._get_ssl_context(req) is ctx
 
 
 async def test___get_ssl_context4(loop) -> None:
-    ctx = ssl.SSLContext()
+    ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     conn = aiohttp.TCPConnector(loop=loop, ssl=ctx)
     req = mock.Mock()
     req.is_ssl.return_value = True
@@ -1289,7 +1321,7 @@ async def test___get_ssl_context4(loop) -> None:
 
 
 async def test___get_ssl_context5(loop) -> None:
-    ctx = ssl.SSLContext()
+    ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     conn = aiohttp.TCPConnector(loop=loop, ssl=ctx)
     req = mock.Mock()
     req.is_ssl.return_value = True
@@ -1301,7 +1333,7 @@ async def test___get_ssl_context6(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     req = mock.Mock()
     req.is_ssl.return_value = True
-    req.ssl = None
+    req.ssl = True
     assert conn._get_ssl_context(req) is conn._make_ssl_context(True)
 
 
@@ -1310,14 +1342,14 @@ async def test_close_twice(loop) -> None:
 
     conn = aiohttp.BaseConnector(loop=loop)
     conn._conns[1] = [(proto, object())]
-    conn.close()
+    await conn.close()
 
     assert not conn._conns
     assert proto.close.called
     assert conn.closed
 
     conn._conns = "Invalid"  # fill with garbage
-    conn.close()
+    await conn.close()
     assert conn.closed
 
 
@@ -1325,7 +1357,7 @@ async def test_close_cancels_cleanup_handle(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     conn._release(1, mock.Mock(should_close=False))
     assert conn._cleanup_handle is not None
-    conn.close()
+    await conn.close()
     assert conn._cleanup_handle is None
 
 
@@ -1334,7 +1366,7 @@ async def test_close_abort_closed_transports(loop) -> None:
 
     conn = aiohttp.BaseConnector(loop=loop)
     conn._cleanup_closed_transports.append(tr)
-    conn.close()
+    await conn.close()
 
     assert not conn._cleanup_closed_transports
     assert tr.abort.called
@@ -1344,7 +1376,7 @@ async def test_close_abort_closed_transports(loop) -> None:
 async def test_close_cancels_cleanup_closed_handle(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, enable_cleanup_closed=True)
     assert conn._cleanup_closed_handle is not None
-    conn.close()
+    await conn.close()
     assert conn._cleanup_closed_handle is None
 
 
@@ -1393,7 +1425,7 @@ async def test_connect_with_limit(loop, key) -> None:
     await asyncio.sleep(0)
     assert acquired
     await task
-    conn.close()
+    await conn.close()
 
 
 async def test_connect_queued_operation_tracing(loop, key) -> None:
@@ -1439,7 +1471,7 @@ async def test_connect_queued_operation_tracing(loop, key) -> None:
     await asyncio.sleep(0.01)
     connection1.release()
     await task
-    conn.close()
+    await conn.close()
 
 
 async def test_connect_reuseconn_tracing(loop, key) -> None:
@@ -1469,7 +1501,7 @@ async def test_connect_reuseconn_tracing(loop, key) -> None:
     on_connection_reuseconn.assert_called_with(
         session, trace_config_ctx, aiohttp.TraceConnectionReuseconnParams()
     )
-    conn.close()
+    await conn.close()
 
 
 async def test_connect_with_limit_and_limit_per_host(loop, key) -> None:
@@ -1503,7 +1535,7 @@ async def test_connect_with_limit_and_limit_per_host(loop, key) -> None:
     await asyncio.sleep(0)
     assert acquired
     await task
-    conn.close()
+    await conn.close()
 
 
 async def test_connect_with_no_limit_and_limit_per_host(loop, key) -> None:
@@ -1535,7 +1567,7 @@ async def test_connect_with_no_limit_and_limit_per_host(loop, key) -> None:
     await asyncio.sleep(0)
     assert acquired
     await task
-    conn.close()
+    await conn.close()
 
 
 async def test_connect_with_no_limits(loop, key) -> None:
@@ -1567,7 +1599,7 @@ async def test_connect_with_no_limits(loop, key) -> None:
     assert acquired
     connection1.release()
     await task
-    conn.close()
+    await conn.close()
 
 
 async def test_connect_with_limit_cancelled(loop) -> None:
@@ -1595,6 +1627,8 @@ async def test_connect_with_limit_cancelled(loop) -> None:
         await asyncio.wait_for(conn.connect(req, None, ClientTimeout()), 0.01)
     connection.close()
 
+    await conn.close()
+
 
 async def test_connect_with_capacity_release_waiters(loop) -> None:
     def check_with_exc(err):
@@ -1668,7 +1702,7 @@ async def test_connect_with_limit_concurrent(loop) -> None:
         await asyncio.wait(tasks)
 
     await f()
-    conn.close()
+    await conn.close()
 
     assert max_connections == num_connections
 
@@ -1731,7 +1765,7 @@ async def test_close_with_acquired_connection(loop) -> None:
     connection = await conn.connect(req, None, ClientTimeout())
 
     assert 1 == len(conn._acquired)
-    conn.close()
+    await conn.close()
     assert 0 == len(conn._acquired)
     assert conn.closed
     proto.close.assert_called_with()
@@ -1750,26 +1784,26 @@ async def test_limit_property(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, limit=15)
     assert 15 == conn.limit
 
-    conn.close()
+    await conn.close()
 
 
 async def test_limit_per_host_property(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, limit_per_host=15)
     assert 15 == conn.limit_per_host
 
-    conn.close()
+    await conn.close()
 
 
 async def test_limit_property_default(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     assert conn.limit == 100
-    conn.close()
+    await conn.close()
 
 
 async def test_limit_per_host_property_default(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     assert conn.limit_per_host == 0
-    conn.close()
+    await conn.close()
 
 
 async def test_force_close_and_explicit_keep_alive(loop) -> None:
@@ -1910,16 +1944,16 @@ async def test_tcp_connector(aiohttp_client, loop) -> None:
     assert r.status == 200
 
 
-@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires unix socket")
+@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires UNIX sockets")
 async def test_unix_connector_not_found(loop) -> None:
-    connector = aiohttp.UnixConnector("/" + uuid.uuid4().hex, loop=loop)
+    connector = aiohttp.UnixConnector("/" + uuid.uuid4().hex)
 
     req = ClientRequest("GET", URL("http://www.python.org"), loop=loop)
     with pytest.raises(aiohttp.ClientConnectorError):
         await connector.connect(req, None, ClientTimeout())
 
 
-@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires unix socket")
+@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires UNIX sockets")
 async def test_unix_connector_permission(loop) -> None:
     loop.create_unix_connection = make_mocked_coro(raise_exception=PermissionError())
     connector = aiohttp.UnixConnector("/" + uuid.uuid4().hex, loop=loop)
@@ -1967,13 +2001,18 @@ async def test_default_use_dns_cache() -> None:
     assert conn.use_dns_cache
 
 
+async def test_ssl_none() -> None:
+    conn = aiohttp.TCPConnector(ssl=None)
+    assert conn._ssl is True
+
+
 async def test_resolver_not_called_with_address_is_ip(loop) -> None:
     resolver = mock.MagicMock()
     connector = aiohttp.TCPConnector(resolver=resolver)
 
     req = ClientRequest(
         "GET",
-        URL("http://127.0.0.1:{}".format(unused_port())),
+        URL(f"http://127.0.0.1:{unused_port()}"),
         loop=loop,
         response_class=mock.Mock(),
     )
@@ -2002,27 +2041,32 @@ async def test_tcp_connector_raise_connector_ssl_error(
     session = aiohttp.ClientSession(connector=conn)
     url = srv.make_url("/")
 
-    if PY_37:
-        err = aiohttp.ClientConnectorCertificateError
-    else:
-        err = aiohttp.ClientConnectorSSLError
-    with pytest.raises(err) as ctx:
+    with pytest.raises(aiohttp.ClientConnectorCertificateError) as ctx:
         await session.get(url)
 
-    if PY_37:
-        assert isinstance(ctx.value, aiohttp.ClientConnectorCertificateError)
-        assert isinstance(ctx.value.certificate_error, ssl.SSLError)
-    else:
-        assert isinstance(ctx.value, aiohttp.ClientSSLError)
-        assert isinstance(ctx.value.os_error, ssl.SSLError)
+    assert isinstance(ctx.value, aiohttp.ClientConnectorCertificateError)
+    assert isinstance(ctx.value.certificate_error, ssl.SSLError)
 
     await session.close()
 
 
+@pytest.mark.parametrize(
+    "host",
+    (
+        pytest.param("127.0.0.1", id="ip address"),
+        pytest.param("localhost", id="domain name"),
+        pytest.param("localhost.", id="fully-qualified domain name"),
+        pytest.param(
+            "localhost...", id="fully-qualified domain name with multiple trailing dots"
+        ),
+        pytest.param("príklad.localhost.", id="idna fully-qualified domain name"),
+    ),
+)
 async def test_tcp_connector_do_not_raise_connector_ssl_error(
     aiohttp_server,
     ssl_ctx,
     client_ssl_ctx,
+    host,
 ) -> None:
     async def handler(request):
         return web.Response()
@@ -2034,10 +2078,33 @@ async def test_tcp_connector_do_not_raise_connector_ssl_error(
     port = unused_port()
     conn = aiohttp.TCPConnector(local_addr=("127.0.0.1", port))
 
+    # resolving something.localhost with the real DNS resolver does not work on macOS, so we have a stub.
+    async def _resolve_host(host, port, traces=None):
+        return [
+            {
+                "hostname": host,
+                "host": "127.0.0.1",
+                "port": port,
+                "family": socket.AF_INET,
+                "proto": 0,
+                "flags": socket.AI_NUMERICHOST,
+            },
+            {
+                "hostname": host,
+                "host": "::1",
+                "port": port,
+                "family": socket.AF_INET,
+                "proto": 0,
+                "flags": socket.AI_NUMERICHOST,
+            },
+        ]
+
+    conn._resolve_host = _resolve_host
+
     session = aiohttp.ClientSession(connector=conn)
     url = srv.make_url("/")
 
-    r = await session.get(url, ssl=client_ssl_ctx)
+    r = await session.get(url.with_host(host), ssl=client_ssl_ctx)
 
     r.release()
     first_conn = next(iter(conn._conns.values()))[0][0]
@@ -2051,7 +2118,7 @@ async def test_tcp_connector_do_not_raise_connector_ssl_error(
     r.close()
 
     await session.close()
-    conn.close()
+    await conn.close()
 
 
 async def test_tcp_connector_uses_provided_local_addr(aiohttp_server) -> None:
@@ -2075,10 +2142,9 @@ async def test_tcp_connector_uses_provided_local_addr(aiohttp_server) -> None:
     assert first_conn.transport.get_extra_info("sockname") == ("127.0.0.1", port)
     r.close()
     await session.close()
-    conn.close()
+    await conn.close()
 
 
-@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires UNIX sockets")
 async def test_unix_connector(unix_server, unix_sockname) -> None:
     async def handler(request):
         return web.Response()
@@ -2161,10 +2227,27 @@ class TestDNSCacheTable:
         dns_cache_table.add("localhost", ["127.0.0.1"])
         assert not dns_cache_table.expired("localhost")
 
-    async def test_expired_ttl(self, loop) -> None:
-        dns_cache_table = _DNSCacheTable(ttl=0.01)
+    def test_expired_ttl(self, monkeypatch: pytest.MonkeyPatch) -> None:
+        dns_cache_table = _DNSCacheTable(ttl=1)
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 1)
         dns_cache_table.add("localhost", ["127.0.0.1"])
-        await asyncio.sleep(0.02)
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 2)
+        assert not dns_cache_table.expired("localhost")
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 3)
+        assert dns_cache_table.expired("localhost")
+
+    def test_never_expire(self, monkeypatch: pytest.MonkeyPatch) -> None:
+        dns_cache_table = _DNSCacheTable(ttl=None)
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 1)
+        dns_cache_table.add("localhost", ["127.0.0.1"])
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 10000000)
+        assert not dns_cache_table.expired("localhost")
+
+    def test_always_expire(self, monkeypatch: pytest.MonkeyPatch) -> None:
+        dns_cache_table = _DNSCacheTable(ttl=0)
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 1)
+        dns_cache_table.add("localhost", ["127.0.0.1"])
+        monkeypatch.setattr("aiohttp.connector.monotonic", lambda: 1.00001)
         assert dns_cache_table.expired("localhost")
 
     def test_next_addrs(self, dns_cache_table) -> None:
@@ -2260,3 +2343,5 @@ async def test_connector_does_not_remove_needed_waiters(loop, key) -> None:
         await_connection_and_check_waiters(),
         allow_connection_and_add_dummy_waiter(),
     )
+
+    await connector.close()
